﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using HarmonyLib;
using Enums;
using System.Configuration;
using UnityEngine.UIElements;
using System.Threading.Tasks;


public class FullBuildingDrops : Mod
{
    private Harmony instance;
    private static GameObject modgui;

    public const String ModName = "FullMaterialDrops";
    public const String ModID = "deezkeyz.keep-materials-from-constructions";
    public static bool keepMudBricks = false;

    public void Start()
    {
        modgui = new GameObject("ModGUI");
        modgui.AddComponent<ModGUI>();

        instance = new Harmony(ModID);
        instance.PatchAll(Assembly.GetExecutingAssembly());

        if(PlayerPrefs.HasKey($"{ModName}:keepMudBricks"))
        {
            keepMudBricks = bool.Parse(PlayerPrefs.GetString($"{ModName}:keepMudBricks"));
        }

        Debug.Log($"Mod {ModName} has been loaded!");
    }


    public void OnModUnload()
    {
        if(modgui != null) { 
            modgui.GetComponent<ModGUI>().showUI = false;
            Destroy(modgui);
        }
        instance?.UnpatchAll(instance.Id);

        PlayerPrefs.SetString($"{ModName}:keepMudBricks", keepMudBricks.ToString());
        PlayerPrefs.Save();
        Debug.Log($"Mod {ModName} has been unloaded!");
    }

    [HarmonyPatch(typeof(SaveGame))]
    [HarmonyPatch("Load", new Type[] { })]
    internal class OnSaveGameLoadedPatch {
        [HarmonyPostfix]
        static async void Postfix()
        {
            await Task.Delay(1000);
            modgui = new GameObject("ModGUI");
            modgui.AddComponent<ModGUI>();
        }
    }

    [HarmonyPatch(typeof(Construction))]
    [HarmonyPatch("CreateItemsOnDestroy")]
    internal class DestroyConstructionPatch
    {
        [HarmonyPrefix]
        static bool Prefix(Construction __instance, ItemID skip_item)
        {            
            GameObject prefab = GreenHellGame.Instance.GetPrefab(__instance.m_Info.m_ID.ToString() + "Ghost");
            if((bool)prefab && __instance.ReplIsOwner())
            {
                foreach(GhostStep step in prefab.GetComponent<ConstructionGhost>().m_Steps)
                {
                    foreach(GhostSlot slot in step.m_Slots)
                    {
                        if(!slot.m_ItemName.ToLower().Contains("mud_from_water"))
                        {
                            if(slot.m_ItemName.ToLower().Contains("mud_to_build") && !keepMudBricks) return false;

                            ItemID itemID = (ItemID)Enum.Parse(typeof(ItemID), slot.m_ItemName);
                            if(itemID != skip_item)
                            {
                                Vector3 position = ((__instance.m_Info.m_ID == ItemID.Campfire && (bool)slot.gameObject.GetComponent<BoxCollider>()) ? slot.gameObject.GetComponent<BoxCollider>().center : slot.transform.localPosition);
                                ItemsManager.Get().CreateItem(itemID, im_register: true, __instance.transform.TransformPoint(position), Quaternion.identity, tempIgnoreCollisionWithPlayer: true);
                            }
                        }
                    }
                }
            }
            if(!__instance.ReplIsOwner())
            {
                return false;
            }
            List<SleighStand> stands = Traverse.Create(__instance).Field("m_SleighStands").GetValue <List<SleighStand>>();

            foreach(SleighStand sleighStand in stands)
            {
                sleighStand.CreateItemsOnDestroy();
            }

            return false;
        }
    }

}



class ModGUI : MonoBehaviour
{
    public bool showUI = false;
    private GUIStyle labelStyle;
    string sanityChange = "0";

    private void Update()
    {
        if(Input.GetKeyDown(KeyCode.F7))
        {
            if(!showUI)
            {
                sanityChange = PlayerConditionModule.Get().m_Sanity.ToString();
                EnableCursor(true);
            }

            showUI = !showUI;

            if(!showUI)
            {
                EnableCursor(false);
            }
        }
    }


    private void EnableCursor(bool enabled = false) 
    {
        CursorManager.Get().ShowCursor(enabled, false);
        Player player = Player.Get();

        if(enabled)
        {
            player.BlockMoves();
            player.BlockRotation();
            player.BlockInspection();
        }
        else
        {
            player.UnblockMoves();
            player.UnblockRotation();
            player.UnblockInspection();
        }
    }
    private void OnGUI()
    {
        if(showUI)
        {
            if(labelStyle == null)
            {
                labelStyle = new GUIStyle(GUI.skin.label);
                labelStyle.fontSize = 14;
                labelStyle.fontStyle = FontStyle.Bold;
                labelStyle.alignment = TextAnchor.MiddleCenter;
            }

            Vector2 screenMidPoint = new Vector2(Screen.width * .5f, Screen.height * .5f);

            GUI.Box(new Rect(screenMidPoint.x - 150f, screenMidPoint.y - 75f, 300f, 100f), "", GUI.skin.window);
            GUI.Label(new Rect(screenMidPoint.x -140, screenMidPoint.y - 77f, 280f, 20f), "Keep Constructionmaterials Settings", labelStyle);

            FullBuildingDrops.keepMudBricks = GUI.Toggle(new Rect(screenMidPoint.x - 130, screenMidPoint.y - 50f, 130f, 40f), FullBuildingDrops.keepMudBricks, "keep Mudbricks", GUI.skin.toggle);

            // Button
            if(GUI.Button(new Rect(screenMidPoint.x - 50, screenMidPoint.y -10f, 100f, 20f), "Close"))
            {
                PlayerPrefs.Save();
                showUI = false;
                EnableCursor(false);
            }

        }
    }
}



public static class ReflectiveEnumerator
{
    public static IEnumerable<Type> GetDerivedTypes<T>() where T : class
    {
        var assembly = Assembly.GetAssembly(typeof(T));
        foreach(var type in assembly.GetTypes().Where(myType => myType.IsClass &&
                                                                 myType.IsSubclassOf(typeof(T))))
        {
            yield return type;
        }
    }
}
